import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_maps/maps.dart';

import '../data_classes.dart';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.sub_div_id, this.thickness, this.color, this.savedColor);

  final String sub_div_id;
  final int thickness;
  Color color;
  final Color savedColor;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements,
    required this.onSelectionChanged,
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;
  final void Function(int selectedIndex) onSelectionChanged; // Callback function

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  late MapShapeSource dataSource;
  int selectedIndex = -1;
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<SubDiv> subdivisions = <SubDiv>[];
  int count = 0;


  @override
  void initState() {
    super.initState();
    // Create list of all subdivisions
    for (Measurement measurement in widget.measurements) {
      for (SubDiv subdivision in measurement.subDivs) {
        subdivisions.add(subdivision);
        count++;
      }
    };

    // Initialise data source
    dataSource = MapShapeSource.memory(
      widget.relation,
      shapeDataField: 'sub_div_id',
      dataCount: count,
      primaryValueMapper: (int index) =>
      subdivisions[index].sub_div_id,
      shapeColorValueMapper: (int index) =>
      subdivisions[index].avgThickness,
      shapeColorMappers: const [
      MapColorMapper(
          from: 0,
          to: 4,
          color: Color(0xFFff0000),
          text: '{0},{25}'),
      MapColorMapper(
          from: 4,
          to: 7,
          color: Color(0xffff6a00),
          text: '75'),
      MapColorMapper(
          from: 7,
          to: 10,
          color: Color(0xFFb1ff00),
          text: '150'),
      MapColorMapper(
          from: 10,
          to: 400,
          color: Color(0xff4fa8d7),
          text: '400'),
        ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMapsTheme(
          data: SfMapsThemeData( // Shape selection behavior
            selectionColor: Colors.orange,
            selectionStrokeWidth: 3,
            selectionStrokeColor: Colors.red[900],
          ),
          child: SfMaps(
            layers: [
              MapShapeLayer(
                source: dataSource,
                //zoomPanBehavior: _zoomPanBehavior,
                strokeColor: Colors.blue.shade50,
                // Shape selection
                selectedIndex: selectedIndex,
                onSelectionChanged: (int index) {
                  setState(() { // Shape selection
                    selectedIndex = index;
                  });
                  widget.onSelectionChanged(selectedIndex);
                },
              ),
            ],
          ),
        ),
      ],
    );
  }
}